#!/usr/bin/env python3

"""
Export application trace events for app startup.
"""

import argparse
import os

from datetime import datetime
from pprint import pprint

from babeltrace import TraceCollection
# Install Python bindings for libbabeltrace:
#     $ sudo add-apt-repository ppa:ted/babeltrace-python
#     $ sudo apt-get update
#     $ sudo apt-get install babeltrace python3-libbabeltrace

from pymongo import MongoClient
# Install Python MongoDB client:
#     $ sudo apt-get install python3-pymongo


def parse_traces(trace_path):
    """
    Read binary traces and convert to dict of
    tracepoint names and timestamps.  Timestamp format is:
    epoch secs + nanosecs.
    """
    
    # Tracepoints to include for app startup
    tracepoints = (
        'upstart_app_launch:libual_start_message_sent',
        'upstart_app_launch:libual_start',
        'upstart_app_launch:exec_pre_exec',
        'upstart_app_launch:libual_start_message_callback',
    )

    # Build up the traces collection
    traces = TraceCollection()
    ret = traces.add_trace(trace_path, 'ctf')
    if ret is None:
        raise IOError('Error adding trace')

    # Record the timestamps
    timestamps = {}
    for event in traces.events:
        # We only want the event if it's the first occurance to
        # handle possibly messy traces.
        if event.name not in timestamps:
            # Keep only the timestamps we care about by event name
            if event.name in tracepoints:
                timestamps[event.name] = event.timestamp
    
    return timestamps


def send_to_db(host, port, doc):
    client = MongoClient(host, port)
    db = client.app_startup_benchmarks
    posts = db.posts
    post_id = posts.insert(doc)
    return post_id


if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('app_id', help='application identifier')
    parser.add_argument('path', help='path to trace')
    args = parser.parse_args()

    doc = dict(
        app_id = args.app_id,
        android_serial = os.environ.get('ANDROID_SERIAL'),
        image_type = os.environ.get('IMAGE_TYPE'),
        build_num = os.environ.get('BUILD_NUMBER'),
        build_id = os.environ.get('BUILD_ID'),
        job_name = os.environ.get('JOB_NAME'),
        node_name = os.environ.get('NODE_NAME'),
        datetime = datetime.utcnow(),
        tracepoints = parse_traces(args.path),
    )

    pprint(doc)

    send_to_db(os.environ.get('BENCHMARKDB_HOST'),
               os.environ.get('BECNHMARKDB_PORT'), doc)
